//
//  BudgetView.swift
//  ydnab
//
//  Created by Andrea Franceschini on 23/09/2020.
//

import SwiftUI

struct BudgetCategoryCell: View {
    @State var category: BudgetCategory
    @State var amountText: String
    @State var color: Color

    var body: some View {
        HStack {
            Text(category.name)
            Spacer()
            Text(amountText)
                .bold()
                .foregroundColor(color)
        }
    }
}

struct BudgetSectionCell: View {
    @State var section: BudgetSection
    @State var amountText: String
    @State var color: Color

    @Environment(\.editMode) var editMode

    var body: some View {
        HStack {
            Text(section.name)
                .textCase(.none)
            Spacer()
            if editMode?.wrappedValue == .inactive {
                Text(amountText)
                    .foregroundColor(color)
            }
        }
        .padding(.vertical, 11)
    }
}

struct YearAndMonthPicker: View {
    @Binding var month: Int
    @Binding var monthName: String
    @Binding var year: Int
    @State var locale: Locale

    private var monthNames: [String] {
        var c = Calendar(identifier: .gregorian)
        c.locale = locale
        return c.monthSymbols
    }

    var columns: [GridItem] = [
        GridItem(spacing: 8),
        GridItem(spacing: 8),
        GridItem(spacing: 8)
    ]

    var body: some View {
        VStack {
            HStack {
                Button(action: { year -= 1 }) {
                    Image(systemName: "chevron.backward.square")
                        .imageScale(.large)
                }
                Spacer()
                Text(String(year)).bold()
                Spacer()
                Button(action: { year += 1 }) {
                    Image(systemName: "chevron.forward.square")
                        .imageScale(.large)
                }
            }

            GeometryReader { geometry in
                LazyVGrid(columns: columns, spacing: 4) {
                    ForEach(monthNames.indices) { i in
                        Button(action: {
                                month = i
                                monthName = monthNames[i]
                        }, label: {
                            Text(monthNames[i])
                                .frame(minWidth: geometry.size.width / 3,
                                       idealWidth: geometry.size.width / 3,
                                       maxWidth: geometry.size.width / 3,
                                       minHeight: geometry.size.height / 4,
                                       idealHeight: geometry.size.height / 4,
                                       maxHeight: geometry.size.height / 4,
                                       alignment: .center)
                        })
                        .background(month == i ? Color("bgActive") : Color("bgInactive"))
                        .foregroundColor(month == i ? Color.white : Color("AccentColor"))
                    }
                }
            }
        }
        .padding(.bottom, 22)
    }
}

struct BudgetViewSummary: View {
    @Binding var budget: BudgetInfo
    @Binding var month: Int
    @Binding var monthName: String
    @Binding var year: Int

    @State private var showMonthPicker: Bool = false

    @State private var showNotImplemented: Bool = false

    var body: some View {
        VStack {
            HStack {
                Button(action: { withAnimation() { showMonthPicker.toggle() } } ) {
                    Text("\(monthName) \(String(year))")
                    Image(systemName: showMonthPicker ? "chevron.up" : "chevron.down")
                }
                Spacer()
                Button(action: { showNotImplemented.toggle() }) { // TODO: Implement this
                    Image(systemName: "bolt")
                }
                Button(action: { showNotImplemented.toggle() }) { // TODO: Implement this
                    Image(systemName: "gear")
                }.padding(.leading, 11)
            }
            .padding(.vertical, 11)

            if showMonthPicker {
                YearAndMonthPicker(month: $month, monthName: $monthName, year: $year, locale: Locale(identifier: budget.localeIdentifier))
            }

            HStack {
                Text("To budget") // Or "overbudgeted"
                Spacer()
                Text("$ 0.00")
            }

            .alert(isPresented: $showNotImplemented) {
                Alert(title: Text("Not Implemented!"))
            }
        }
        .padding(.vertical, 11)
    }
}

struct BudgetView: View {
    @State var budget: BudgetInfo
    @Binding var currentBudgetId: UUID?
    @State private var month: Int = 0
    @State private var monthName: String
    @State private var year: Int = Calendar(identifier: .gregorian).component(.year, from: Date())

    @AppStorage("lastLoadedBudgetId") private var lastLoadedBudgetId = ""

    init(budget: BudgetInfo, currentBudgetId: Binding<UUID?>) {
        _budget = .init(initialValue: budget)
        _currentBudgetId = currentBudgetId
        var cal = Calendar(identifier: .gregorian)
        cal.locale = Locale(identifier: self._budget.wrappedValue.localeIdentifier)
        let now = Date()
        _month = .init(initialValue: cal.component(.month, from: now))
        _monthName = .init(initialValue: cal.monthSymbols[self._month.wrappedValue])
        _year = .init(initialValue: cal.component(.year, from: now))
    }

    func formatAmount(_ amount: NSNumber) -> String? {
        let f = NumberFormatter()
        f.locale = Locale(identifier: budget.localeIdentifier)
        f.numberStyle = .currency
        return f.string(from: amount)
    }

    var body: some View {
        VStack {
            BudgetViewSummary(budget: $budget, month: $month, monthName: $monthName, year: $year)
                .padding([.leading, .trailing], 16)

            List {
                ForEach(budget.sections) { section in
                    if !section.hidden {
                        Section(header: BudgetSectionCell(section: section,
                                                          amountText: formatAmount(0) ?? "--",
                                                          color: 0 < 0 ? Color("negativeAmount") : Color("positiveAmount"))
                        ) {
                            ForEach(section.categories) { category in
                                if !category.hidden {
                                    BudgetCategoryCell(category: category,
                                                       amountText: formatAmount(0) ?? "--",
                                                       color: 0 < 0 ? Color("negativeAmount") : Color("positiveAmount")
                                    )
                                }
                            }
                            .onMove { (indexSet, s) in
                                print(indexSet, s)
                            }
                        }
                    }
                }
                .onMove { (indexSet, s) in
                    print(indexSet, s)
                }
            }
            .listStyle(PlainListStyle())
            .navigationBarTitle(budget.name, displayMode: .inline)
            .toolbar {
                EditButton()
            }
            .onAppear() {
                print("BudgetView Appears")
                currentBudgetId = budget.id
                lastLoadedBudgetId = budget.id.uuidString
            }
        }
    }
}

struct BudgetView_Previews: PreviewProvider {
    static let budget: [BudgetSection] = [
        BudgetSection(name: "Everyday Expenses", categories: [
            BudgetCategory(name: "Groceries"),
            BudgetCategory(name: "Eating out"),
            BudgetCategory(name: "Medical"),
            BudgetCategory(name: "Clothing"),
            BudgetCategory(name: "Household goods")
        ]),
        BudgetSection(name: "Travel", categories: [
            BudgetCategory(name: "Transport"),
            BudgetCategory(name: "Fuel"),
            BudgetCategory(name: "Accommodation")
        ]),
        BudgetSection(name: "Rainy Days", categories: [
            BudgetCategory(name: "Emergencies"),
            BudgetCategory(name: "Car insurance")
        ]),
        BudgetSection(name: "Monthly Expenses", categories: [
            BudgetCategory(name: "Rent"),
            BudgetCategory(name: "Mobile")
        ]),
        BudgetSection(name: "Savings Goals", categories: [
            BudgetCategory(name: "Savings"),
            BudgetCategory(name: "Holidays")
        ])
    ]
    @State static var currentBudgetId: UUID?

    static var previews: some View {
        NavigationView {
            BudgetView(budget: BudgetInfo(name: "Default Budget", sections: budget),
                       currentBudgetId: $currentBudgetId)
        }
        .preferredColorScheme(.light)

        //.environment(\.layoutDirection, .rightToLeft)
    }
}

